
#ifndef COLOR_HPP
#define COLOR_HPP

namespace gar3d {

	typedef float ColorChannel;

	enum standardColor {
		black=0,
		blue=1,
		green=2,
		cyan=3,
		red=4,
		magenta=5,
		yellow=6,
		white=7,
		gray=8,
		grey=8,
	};

	class Color {
	public:
		Color() 
			: Color(1, 1, 1) { }
		Color(ColorChannel r, ColorChannel g, ColorChannel b)
			: _r(clamp01(r)), _g(clamp01(g)), _b(clamp01(b)) {}
		Color(const Color& other)
			: _r(other._r), _g(other._g), _b(other._b) {}
		Color(standardColor stdcol) {
			if (stdcol == gray || stdcol == grey) {
				_r = .5; _g = .5; _b = .5;
			}
			else {
				_r = clamp01(stdcol & standardColor::red);
				_g = clamp01(stdcol & standardColor::green);
				_b = clamp01(stdcol & standardColor::blue);
			}
		}

		ColorChannel red() const { return _r; }
		ColorChannel green() const { return _g; }
		ColorChannel blue() const { return _b; }
		ColorChannel red(ColorChannel r) { return _r = clamp01(r); }
		ColorChannel green(ColorChannel g) { return _g = clamp01(g); }
		ColorChannel blue(ColorChannel b) { return _b = clamp01(b); }

		Color& operator =(const Color& other) {
			_r = other._r;
			_g = other._g;
			_b = other._b;
			return *this;
		}
	private:
		ColorChannel clamp01(ColorChannel n) { return (n > 1) ? 1 : (n < 0) ? 0 : n; }
		ColorChannel _r;
		ColorChannel _g;
		ColorChannel _b;
	};

}  // namespace gar3d

#endif  // COLOR_HPP
