#ifndef INPUTHANDLE_HPP
#define INPUTHANDLE_HPP

#include <iostream>
#include <string>
#include "SDL_GLM_INCLUDES.hpp"

namespace gar3d {

	enum mouseButton : int {
		mouse_left = SDL_BUTTON_LEFT,
		mouse_right = SDL_BUTTON_RIGHT,
		mouse_middle = SDL_BUTTON_MIDDLE,
	};

	enum Key : int {
		letter_a = SDL_SCANCODE_A,
		letter_b = SDL_SCANCODE_B,
		letter_c = SDL_SCANCODE_C,
		letter_d = SDL_SCANCODE_D,
		letter_e = SDL_SCANCODE_E,
		letter_f = SDL_SCANCODE_F,
		letter_g = SDL_SCANCODE_G,
		letter_h = SDL_SCANCODE_H,
		letter_i = SDL_SCANCODE_I,
		letter_j = SDL_SCANCODE_J,
		letter_k = SDL_SCANCODE_K,
		letter_l = SDL_SCANCODE_L,
		letter_m = SDL_SCANCODE_M,
		letter_n = SDL_SCANCODE_N,
		letter_o = SDL_SCANCODE_O,
		letter_p = SDL_SCANCODE_P,
		letter_q = SDL_SCANCODE_Q,
		letter_r = SDL_SCANCODE_R,
		letter_s = SDL_SCANCODE_S,
		letter_t = SDL_SCANCODE_T,
		letter_u = SDL_SCANCODE_U,
		letter_v = SDL_SCANCODE_V,
		letter_w = SDL_SCANCODE_W,
		letter_x = SDL_SCANCODE_X,
		letter_y = SDL_SCANCODE_Y,
		letter_z = SDL_SCANCODE_Z,

		arrow_up = SDL_SCANCODE_UP,
		arrow_down = SDL_SCANCODE_DOWN,
		arrow_left = SDL_SCANCODE_LEFT,
		arrow_right = SDL_SCANCODE_RIGHT,

		number_0 = SDL_SCANCODE_0,
		number_1 = SDL_SCANCODE_1,
		number_2 = SDL_SCANCODE_2,
		number_3 = SDL_SCANCODE_3,
		number_4 = SDL_SCANCODE_4,
		number_5 = SDL_SCANCODE_5,
		number_6 = SDL_SCANCODE_6,
		number_7 = SDL_SCANCODE_7,
		number_8 = SDL_SCANCODE_8,
		number_9 = SDL_SCANCODE_9,

		numpad_0 = SDL_SCANCODE_KP_0,
		numpad_1 = SDL_SCANCODE_KP_1,
		numpad_2 = SDL_SCANCODE_KP_2,
		numpad_3 = SDL_SCANCODE_KP_3,
		numpad_4 = SDL_SCANCODE_KP_4,
		numpad_5 = SDL_SCANCODE_KP_5,
		numpad_6 = SDL_SCANCODE_KP_6,
		numpad_7 = SDL_SCANCODE_KP_7,
		numpad_8 = SDL_SCANCODE_KP_8,
		numpad_9 = SDL_SCANCODE_KP_9,

		symbol_minus = SDL_SCANCODE_MINUS,
		symbol_equals = SDL_SCANCODE_EQUALS,
		symbol_left_bracket = SDL_SCANCODE_LEFTBRACKET,
		symbol_right_bracket = SDL_SCANCODE_RIGHTBRACKET,
		symbol_backslash = SDL_SCANCODE_BACKSLASH,
		symbol_semicolon = SDL_SCANCODE_SEMICOLON,
		symbol_apostrophe = SDL_SCANCODE_APOSTROPHE,
		symbol_comma = SDL_SCANCODE_COMMA,
		symbol_period = SDL_SCANCODE_PERIOD,
		symbol_slash = SDL_SCANCODE_SLASH,
		symbol_grave = SDL_SCANCODE_GRAVE,

		symbol_numpad_slash = SDL_SCANCODE_SLASH,
		symbol_numpad_asterisk = SDL_SCANCODE_SLASH,
		symbol_numpad_minus = SDL_SCANCODE_SLASH,
		symbol_numpad_plus = SDL_SCANCODE_SLASH,

		left_shift = SDL_SCANCODE_LSHIFT,
		right_shift = SDL_SCANCODE_RSHIFT,
		left_control = SDL_SCANCODE_LCTRL,
		right_control = SDL_SCANCODE_RCTRL,
	};

	class InputHandle {
		friend class Gar3D;
		const SDL_Scancode quitkey = SDL_SCANCODE_ESCAPE;
	public:
		InputHandle();
		~InputHandle();
		bool mouseButtonHeld(mouseButton button) const;
		bool mouseButtonReleased(mouseButton button) const;
		bool mouseButtonPressed(mouseButton button) const;
		bool buttonHeld(Key code) const;
		bool buttonReleased(Key code) const;
		bool buttonPressed(Key code) const;
		void mousePosition(int &x, int &y) const;
		void getMouseMotion(int &x, int &y) const { x = mouseMotion.x; y = mouseMotion.y; }
		int mouseX() const;
		int mouseY() const;

	private:
		bool userQuit;
		typedef Uint8* KeyboardState;
		glm::vec2 mouseMotion;
		KeyboardState currentKeyboardState;
		KeyboardState previousKeyboardState;
		bool previousMouseButtonState[6];
		bool currentMouseButtonState[6];
		
		void updateState();
		void pollEvents();
		void onKeyDown(SDL_KeyboardEvent&);
		void onKeyUp(SDL_KeyboardEvent&);
		void onMouseMotion(SDL_MouseMotionEvent&);
		void onMouseButtonDown(SDL_MouseButtonEvent&);
		void onMouseButtonUp(SDL_MouseButtonEvent&);
	};

}

#endif // INPUTHANDLE_HPP
