
#ifndef DURATION_CPP
#define DURATION_CPP

#include "Duration.h"

Duration::Duration()
	: Duration(CLK::now()) {}
Duration::Duration(T_P tp)
	: Duration(tp.time_since_epoch().count(), TimeUnit::Nanosecond) {}
Duration::Duration(double t, TimeUnit unit) {
	_nanos = t * nscounts[unit];
	dirtyflags = 0x7e;
}
double Duration::Nanoseconds() {
	return _nanos;
}
double Duration::Microseconds() {
	if (dirtyflags & _flags::micro) {
		_micros = _nanos / 1e3;
		dirtyflags |= (~_flags::micro);
	}
	return _micros;
}
double Duration::Milliseconds() {
	if (dirtyflags & _flags::milli) {
		_millis = _nanos / 1e6;
		dirtyflags |= (~_flags::milli);
	}
	return _millis;
}
double Duration::Seconds() {
	if (dirtyflags & _flags::sec) {
		_secs = _nanos / 1e9;
		dirtyflags |= (~_flags::sec);
	}
	return _secs;
}
double Duration::Minutes() {
	if (dirtyflags & _flags::min) {
		_mins = _nanos / 6e10;
		dirtyflags |= (~_flags::min);
	}
	return _mins;
}
double Duration::Hours() {
	if (dirtyflags & _flags::hour) {
		_hours = _nanos / 3.6e12;
		dirtyflags |= (~_flags::hour);
	}
	return _hours;
}
double Duration::Days() {
	if (dirtyflags & _flags::day) {
		_days = _nanos / 8.64e13;
		dirtyflags |= (~_flags::day);
	}
	return _days;
}
Duration abs(Duration d) {
	return Duration(fabs(d.Nanoseconds()), TimeUnit::Nanosecond);
}
Duration operator -(Duration d) {
	return Duration(-d.Nanoseconds(), TimeUnit::Nanosecond);
}
Duration operator -(Duration d1, Duration d0) {
	return Duration(d0.Nanoseconds() - d0.Nanoseconds(), TimeUnit::Nanosecond);
}
Duration operator +(Duration d1, Duration d0) {
	return Duration(d0.Nanoseconds() + d0.Nanoseconds(), TimeUnit::Nanosecond);
}
Duration operator *(Duration d, double x) {
	return Duration(d.Nanoseconds() * x, TimeUnit::Nanosecond);
}
Duration operator *(double x, Duration d){
	return Duration(d.Nanoseconds() * x, TimeUnit::Nanosecond);
}
Duration operator /(Duration d, double x) {
	return Duration(d.Nanoseconds() / x, TimeUnit::Nanosecond);
}
Duration Duration::operator +=(Duration d) {
	_nanos += d.Nanoseconds();
	dirtyflags = 0x7e;
	return *this;
}
Duration Duration::operator -=(Duration d) {
	_nanos -= d.Nanoseconds();
	dirtyflags = 0x7e;
	return *this;
}
Duration Duration::operator *=(double d) {
	_nanos *= d;
	dirtyflags = 0x7e;
	return *this;
}
Duration Duration::operator /=(double d) {
	_nanos /= d;
	dirtyflags = 0x7e;
	return *this;
}
bool operator ==(Duration tp1, Duration tp2) {
	return tp1.Nanoseconds() == tp2.Nanoseconds();
}
bool operator !=(Duration tp1, Duration tp2) {
	return tp1.Nanoseconds() != tp2.Nanoseconds();
}
bool operator >=(Duration tp1, Duration tp2) {
	return tp1.Nanoseconds() >= tp2.Nanoseconds();
}
bool operator <=(Duration tp1, Duration tp2) {
	return tp1.Nanoseconds() <= tp2.Nanoseconds();
}
bool operator >(Duration tp1, Duration tp2) {
	return tp1.Nanoseconds() > tp2.Nanoseconds();
}
bool operator <(Duration tp1, Duration tp2) {
	return tp1.Nanoseconds() < tp2.Nanoseconds();
}
Duration::operator double() {
	return Seconds();
}

#endif // DURATION_CPP
