#pragma once
#ifndef DURATION_H
#define DURATION_H

#include <chrono>

typedef std::chrono::system_clock CLK;
typedef std::chrono::duration<double, std::nano> DUR;
typedef std::chrono::time_point<CLK, DUR> T_P;

enum TimeUnit {
	Nanosecond = 0,
	Microsecond = 1,
	Millisecond = 2,
	Second = 3,
	Minute = 4,
	Hour = 5,
	Day = 6,
};

class Duration {
public:
	Duration();
	Duration(T_P);
	Duration(double, TimeUnit);
	double Nanoseconds();
	double Microseconds();
	double Milliseconds();
	double Seconds();
	double Minutes();
	double Hours();
	double Days();
	friend Duration abs(Duration);
	friend Duration operator -(Duration);
	friend Duration operator -(Duration, Duration);
	friend Duration operator +(Duration, Duration);
	friend Duration operator *(Duration, double);
	friend Duration operator *(double, Duration);
	friend Duration operator /(Duration, double);
	Duration operator +=(Duration);
	Duration operator -=(Duration);
	Duration operator *=(double);
	Duration operator /=(double);
	friend bool operator ==(Duration, Duration);
	friend bool operator !=(Duration, Duration);
	friend bool operator >=(Duration, Duration);
	friend bool operator <=(Duration, Duration);
	friend bool operator >(Duration, Duration);
	friend bool operator <(Duration, Duration);
	operator double();
private:
	unsigned dirtyflags;
	enum _flags {
		nano = 0x1,	//	nano should never be dirty... hopefully
		micro = 0x2,
		milli = 0x4,
		sec = 0x8,
		min = 0x10,
		hour = 0x20,
		day = 0x40,
	};
	double _nanos;
	double _micros;
	double _millis;
	double _secs;
	double _mins;
	double _hours;
	double _days;
	double nscounts[7]{ 1, 1e3, 1e6, 1e9, 6e10, 3.6e12, 8.64e13 };
};

#endif	//	DURATION_H