#pragma once
#ifndef TIMEPOINT_H
#define TIMEPOINT_H

#include <chrono>
#include "Duration.h"


class Timepoint {
public:
	Timepoint();
	Timepoint(T_P);
	double Nanoseconds();
	double Microseconds();
	double Milliseconds();
	double Seconds();
	double Minutes();
	double Hours();
	double Days();

	static Timepoint Zero() {
		return Timepoint(0);
	}

	friend Duration operator-(Timepoint, Timepoint);
	friend Timepoint operator +(Timepoint, Duration);
	friend Timepoint operator -(Timepoint, Duration);
	Timepoint operator +=(Duration);
	Timepoint operator -=(Duration);
	friend bool operator ==(Timepoint, Timepoint);
	friend bool operator !=(Timepoint, Timepoint);
	friend bool operator >=(Timepoint, Timepoint);
	friend bool operator <=(Timepoint, Timepoint);
	friend bool operator >(Timepoint, Timepoint);
	friend bool operator <(Timepoint, Timepoint);
private:
	unsigned dirtyflags;
	enum _flags {
		nano = 0x1,	//	nano should never be dirty... hopefully
		micro = 0x2,
		milli = 0x4,
		sec = 0x8,
		min = 0x10,
		hour = 0x20,
		day = 0x40,
	};
	double _nanos;
	double _micros;
	double _millis;
	double _secs;
	double _mins;
	double _hours;
	double _days;
	Timepoint(double nanos);
};

#endif	//	TIMEPOINT_H